<?php
/**
 * @file
 * Taxonomy Related bean plugin.
 */

class TaxRelatedBean extends BeanPlugin {

  /**
   * Declares default block settings.
   */
  public function values() {
    return array(
      'filters' => array(
        'vocabulary' => FALSE,
        'content_types' => FALSE,
      ),
      'settings' => array(
        'node_view_mode' => 'full',
        'records_shown' => 5,
      ),
      'more_link' => array(
        'text' => '',
        'path' => '',
      ),
    );
  }

  /**
   * Builds extra settings for the block edit form.
   */
  public function form($bean, $form, &$form_state) {

    // Here we are defining the form.
    $form = array();

    // Select objects for the vocabularies that will be used.
    $vocabulary = taxonomy_get_vocabularies();
    $select_vocabulary_array = array();
    foreach ($vocabulary as $item) {
      $key = $item->vid;
      $value = $item->name;
      $select_vocabulary_array[$key] = $value;
    }

    // Select objects for the node entity bundles that will be used.
    $node_type = node_type_get_types();
    $select_content_types_array = array();
    foreach ($node_type as $item) {
      $key = $item->type;
      $value = $item->name;
      $select_content_types_array[$key] = $value;
    }

    // Building the basic form for individual 'related' bean configuration.
    $form['filters'] = array(
      '#type' => 'fieldset',
      '#tree' => TRUE,
      '#title' => t('Filters'),
    );

    // Determine what node bundles to display.
    $form['filters']['content_types'] = array(
      '#type' => 'select',
      '#title' => t('Content'),
      '#options' => $select_content_types_array,
      '#default_value' => $bean->filters['content_types'],
      '#required' => TRUE,
      '#multiple' => TRUE,
      '#size' => 5,
    );

    // Determine related taxonomy term vocabularies.
    $form['filters']['vocabulary'] = array(
      '#type' => 'select',
      '#title' => t('Vocabulary'),
      '#options' => $select_vocabulary_array,
      '#default_value' => $bean->filters['vocabulary'],
      '#required' => TRUE,
      '#multiple' => TRUE,
      '#size' => 5,
    );

    // Settings fieldset is used for configuring the 'related' bean output.
    $form['settings'] = array(
      '#type' => 'fieldset',
      '#tree' => TRUE,
      '#title' => t('Output'),
    );

    // Select objects for the node view modes that will be used.
    $node_view_modes = array();
    $entity_info = entity_get_info();
    foreach ($entity_info['node']['view modes'] as $key => $value) {
      $node_view_modes[$key] = $value['label'];
    }

    // User select which view mode to use for the node results inside the bean.
    $form['settings']['node_view_mode'] = array(
      '#type' => 'select',
      '#title' => t('Node View Mode'),
      '#options' => $node_view_modes,
      '#default_value' => $bean->settings['node_view_mode'],
      '#required' => TRUE,
      '#multiple' => FALSE,
    );

    // User define a maximum number of nodes to be shown.
    $form['settings']['records_shown'] = array(
      '#type' => 'textfield',
      '#title' => t('Records shown'),
      '#size' => 5,
      '#default_value' => $bean->settings['records_shown'],
      '#element_validate' => array('bean_tax_setting_is_numeric'),
    );

    // Define a "read more" style link to be shown at the bottom of the bean.
    $form['more_link'] = array(
      '#type' => 'fieldset',
      '#tree' => TRUE,
      '#title' => t('More link'),
    );

    // Link text shown on the 'more link' to be defined by user.
    $form['more_link']['text'] = array(
      '#type' => 'textfield',
      '#title' => t('Link text'),
      '#default_value' => $bean->more_link['text'],
    );

    // Actual URL path for the 'more link' defined by user.
    $form['more_link']['path'] = array(
      '#type' => 'textfield',
      '#title' => t('Link path'),
      '#default_value' => $bean->more_link['path'],
    );
    return $form;
  }

  /**
   * Displays the bean.
   */
  public function view($bean, $content, $view_mode = 'default', $langcode = NULL) {

    // We need to make sure that the bean is configured correctly.
    if (!empty($bean->filters['vocabulary']) && !empty($bean->filters['content_types'])) {

      // Define an array of all taxonomy terms in the defined vocabularies.
      $possible_tid = array();
      foreach ($bean->filters['vocabulary'] as $vid) {
        $tree = taxonomy_get_tree($vid);
        foreach ($tree as $term) {
          array_push($possible_tid, $term->tid);
        }
      }

      // Compare possible terms to those attached to the node.
      if ($node = menu_get_object()) {
        $active_nid = $node->nid;
        $node_terms = bean_tax_get_node_terms($node);
        $valid_tid = array();
        foreach ($node_terms as $term) {
          if (in_array($term->tid, $possible_tid)) {
            array_push($valid_tid, $term->tid);
          }
        }

        // Entity field query for nodes of the defined content types (bundle).
        $aggregate = array();
        foreach ($bean->filters['content_types'] as $bundle) {
          $query = new EntityFieldQuery();
          $query->entityCondition('entity_type', 'node');
          $query->entityCondition('bundle', $bundle);
          $query->propertyCondition('status', 1);
          $query->propertyOrderBy('created', 'DESC');
          $result[$bundle] = $query->execute();

          // Store the results in an aggregated array of nodes.
          if (!empty($result[$bundle])) {
            $aggregate = $aggregate + $result[$bundle]['node'];
          }

        }

        // Create a taxonomy related "score" for each result's matching terms.
        $result = array();
        foreach ($aggregate as $node) {
          $node_terms = bean_tax_get_node_terms($node);
          $score = 0;
          // The actual scoring to determine valid taxonomy term matching.
          foreach ($node_terms as $term) {
            if (in_array($term->tid, $valid_tid)) {
              $score++;
            }
          }
          // A score of 1 or greater adds to the array of matching nodes.
          if ($score != 0 && $node->nid != $active_nid) {
            $match['nid'] = $node->nid;
            $match['vid'] = $node->vid;
            $match['type'] = $node->type;
            $match['score'] = $score;
            array_push($result, $match);
          }
        }
        // Invoke comparison function to determine most relative results.
        usort($result, "bean_tax_cmp");
      }

      // Relative nodes initially set to none.
      $rel = 0;
      if (empty($result)) {
        // There are no related nodes. Set Empty array for theme output.
        $content['nodes'] = array();
      }
      else {
        // There are related node results so we process further.
        $i = 1;
        // Invoke node_view on the related nodes inside 'records_shown' limit.
        foreach ($result as $match) {
          if ($i <= $bean->settings['records_shown']) {
            $node = node_load($match['nid'], $match['vid']);
            $content['nodes'][$node->nid] = node_view($node, $bean->settings['node_view_mode']);
            $rel++;
          }
          $i++;
        }
      }
      // No relative nodes to be shown - show user friendly output.
      if ($rel == 0) {
        $content['#markup'] = t('No Results');
      }
    }
    // Invoke the theme function to show the additional information "more link"
    $content['more_link']['#markup'] = theme('bean_tax_more_link', array('text' => $bean->more_link['text'], 'path' => $bean->more_link['path']));
    return $content;
  }
}

